@@ -16,8 +16,6 @@ enum chacha20poly1305_lengths {
 	CHACHA20POLY1305_AUTHTAGLEN = 16
 };
 
-void chacha20poly1305_fpu_init(void);
-
 void chacha20poly1305_encrypt(u8 *dst, const u8 *src, const size_t src_len,
 			      const u8 *ad, const size_t ad_len,
 			      const u64 nonce, const u8 key[CHACHA20POLY1305_KEYLEN]);
@@ -58,7 +56,7 @@ bool __must_check xchacha20poly1305_decrypt(u8 *dst, const u8 *src, const size_t
 static inline bool chacha20poly1305_init_simd(void)
 {
 	bool have_simd = false;
-#if defined(CONFIG_X86_64) && !defined(CONFIG_UML)
+#if defined(CONFIG_X86_64) && !defined(CONFIG_UML) && !defined(CONFIG_PREEMPT_RT_BASE)
 	have_simd = irq_fpu_usable();
 	if (have_simd)
 		kernel_fpu_begin();
@@ -87,7 +85,6 @@ static inline void chacha20poly1305_deinit_simd(bool was_on)
 
 #ifdef DEBUG
 bool chacha20poly1305_selftest(void);
-bool poly1305_selftest(void);
 #endif
 
 #endif /* _WG_CHACHA20POLY1305_H */
