@@ -1,4 +1,7 @@
-/* Copyright (C) 2015-2017 Jason A. Donenfeld <Jason@zx2c4.com>. All Rights Reserved. */
+/* SPDX-License-Identifier: GPL-2.0
+ *
+ * Copyright (C) 2015-2018 Jason A. Donenfeld <Jason@zx2c4.com>. All Rights Reserved.
+ */
 
 #include "version.h"
 #include "device.h"
@@ -6,7 +9,8 @@
 #include "queueing.h"
 #include "ratelimiter.h"
 #include "netlink.h"
-#include "crypto/chacha20poly1305.h"
+#include "crypto/chacha20.h"
+#include "crypto/poly1305.h"
 #include "crypto/blake2s.h"
 #include "crypto/curve25519.h"
 #include "uapi/wireguard.h"
@@ -21,11 +25,12 @@ static int __init mod_init(void)
 {
 	int ret;
 
-	chacha20poly1305_fpu_init();
+	chacha20_fpu_init();
+	poly1305_fpu_init();
 	blake2s_fpu_init();
 	curve25519_fpu_init();
 #ifdef DEBUG
-	if (!routing_table_selftest() || !packet_counter_selftest() || !curve25519_selftest() || !chacha20poly1305_selftest() || !blake2s_selftest() || !ratelimiter_selftest())
+	if (!allowedips_selftest() || !packet_counter_selftest() || !curve25519_selftest() || !chacha20poly1305_selftest() || !poly1305_selftest() || !blake2s_selftest() || !ratelimiter_selftest())
 		return -ENOTRECOVERABLE;
 #endif
 	noise_init();
@@ -34,12 +39,12 @@ static int __init mod_init(void)
 	if (ret < 0)
 		goto err_packet;
 
-	ret = netlink_init();
+	ret = genetlink_init();
 	if (ret < 0)
 		goto err_netlink;
 
 	pr_info("WireGuard " WIREGUARD_VERSION " loaded. See www.wireguard.com for information.\n");
-	pr_info("Copyright (C) 2015-2017 Jason A. Donenfeld <Jason@zx2c4.com>. All Rights Reserved.\n");
+	pr_info("Copyright (C) 2015-2018 Jason A. Donenfeld <Jason@zx2c4.com>. All Rights Reserved.\n");
 
 	return 0;
 
@@ -51,7 +56,7 @@ err_packet:
 
 static void __exit mod_exit(void)
 {
-	netlink_uninit();
+	genetlink_uninit();
 	device_uninit();
 	pr_debug("WireGuard unloaded\n");
 }
